"use strict";
exports.__esModule = true;
var fs_1 = require("fs");
var ts = require("typescript");
var prelude_ts_1 = require("prelude.ts");
/**
 * return a pair:
 * fst => the body of the interface (source code of the implementation)
 * snd => generic type parameters for the interface, or "". (For instance "<T0, T1>")
 *
 * The thinking for the generic type parameters is that you may have a
 * generic scope which doesn't care which is the T. In that case we can type-check
 * against any T. If you do care about the T, you can subclass the scope type to
 * a concrete type.
 */
function parseScopeInterface(iface) {
    var typeParamsInfo = function () {
        return prelude_ts_1.Option.of(iface.typeParameters)
            .filter(function (p) { return p.length > 0; })
            .map(function (p) { return "<" + p.map(function (_, idx) { return "T" + idx; }).join(", ") + ">"; })
            .getOrElse("");
    };
    return prelude_ts_1.Option.of(iface.getText())
        .filter(function (_) { return iface.name.getText() === "Scope"; })
        .map(function (cts) { return prelude_ts_1.Tuple2.of(cts, typeParamsInfo()); });
}
var maybeNodeType = function (sKind) { return function (input) {
    return (input && input.kind === sKind) ? prelude_ts_1.Option.of(input) : prelude_ts_1.Option.none();
}; };
/**
 * @hidden
 */
function maybeSingleNode(nodes) {
    if (nodes && nodes.length === 1) {
        return prelude_ts_1.Option.of(nodes[0]);
    }
    return prelude_ts_1.Option.none();
}
exports.maybeSingleNode = maybeSingleNode;
/**
 * @hidden
 */
exports.maybeCallExpression = maybeNodeType(ts.SyntaxKind.CallExpression);
/**
 * @hidden
 */
exports.maybePropertyAccessExpression = maybeNodeType(ts.SyntaxKind.PropertyAccessExpression);
/**
 * @hidden
 */
exports.maybePropertyAssignment = maybeNodeType(ts.SyntaxKind.PropertyAssignment);
/**
 * @hidden
 */
exports.maybeIdentifier = maybeNodeType(ts.SyntaxKind.Identifier);
/**
 * @hidden
 */
exports.maybeStringLiteral = maybeNodeType(ts.SyntaxKind.StringLiteral);
/**
 * @hidden
 */
exports.maybeObjectLiteralExpression = maybeNodeType(ts.SyntaxKind.ObjectLiteralExpression);
/**
 * @hidden
 */
exports.maybeVariableStatement = maybeNodeType(ts.SyntaxKind.VariableStatement);
/**
 * @hidden
 */
exports.maybeArrowFunction = maybeNodeType(ts.SyntaxKind.ArrowFunction);
/**
 * @hidden
 */
exports.maybeFunctionExpression = maybeNodeType(ts.SyntaxKind.FunctionExpression);
/**
 * @hidden
 */
exports.maybeBlock = maybeNodeType(ts.SyntaxKind.Block);
/**
 * @hidden
 */
exports.maybeReturnStatement = maybeNodeType(ts.SyntaxKind.ReturnStatement);
/**
 * @hidden
 */
exports.maybeAsExpression = maybeNodeType(ts.SyntaxKind.AsExpression);
/**
 * @hidden
 */
exports.maybeArrayLiteralExpression = maybeNodeType(ts.SyntaxKind.ArrayLiteralExpression);
function objectLiteralGetProperty(propName, elts) {
    return elts.find(function (elt) { return exports.maybeIdentifier(elt.name).filter(function (i) { return i.text === propName; }).isSome(); });
}
function getFieldStringLiteralValue(field) {
    return exports.maybePropertyAssignment(field)
        .flatMap(function (pa) { return exports.maybeStringLiteral(pa.initializer); })
        .map(function (ini) { return ini.text; });
}
function objectLiteralGetStringLiteralField(propName, elts) {
    return objectLiteralGetProperty(propName, elts)
        .flatMap(function (p) { return getFieldStringLiteralValue(p); });
}
function parseModalOpen(callExpr) {
    var paramObjectElements = prelude_ts_1.Option.of(callExpr)
        .filter(function (c) { return ["$modal.open", "this.$modal.open"]
        .indexOf(c.expression.getText()) >= 0; })
        .flatMap(function (c) { return maybeSingleNode(c.arguments); })
        .flatMap(function (a) { return exports.maybeObjectLiteralExpression(a); })
        .map(function (o) { return prelude_ts_1.Vector.ofIterable(o.properties); });
    var getField = function (name) {
        return paramObjectElements.flatMap(function (oe) { return objectLiteralGetStringLiteralField(name, oe); });
    };
    var controllerName = getField("controller");
    var rawViewPath = getField("templateUrl");
    var buildCtrlViewInfo = function (rawViewPath, ctrlName) {
        return ({ controllerName: ctrlName, viewPath: rawViewPath });
    };
    return prelude_ts_1.Option.liftA2(buildCtrlViewInfo)(rawViewPath, controllerName);
}
function parseModuleState(prop) {
    var objectLiteralFields = prop.properties
        .map(function (e) { return exports.maybeIdentifier(e.name); })
        .filter(function (i) { return i.isSome(); })
        .map(function (i) { return i.getOrThrow().text; });
    if ((objectLiteralFields.indexOf("url") >= 0) &&
        (objectLiteralFields.indexOf("templateUrl") >= 0) &&
        (objectLiteralFields.indexOf("controller") >= 0)) {
        // seems like I got a state controller/view declaration
        var controllerName = objectLiteralGetStringLiteralField("controller", prelude_ts_1.Vector.ofIterable(prop.properties));
        var rawViewPath = objectLiteralGetStringLiteralField("templateUrl", prelude_ts_1.Vector.ofIterable(prop.properties));
        var buildCtrlViewInfo = function (rawViewPath, ctrlName) {
            return ({ controllerName: ctrlName, viewPath: rawViewPath });
        };
        return prelude_ts_1.Option.liftA2(buildCtrlViewInfo)(rawViewPath, controllerName);
    }
    return prelude_ts_1.Option.none();
}
function parseAngularModule(expr) {
    var callExpr = exports.maybeCallExpression(expr.expression);
    var prop0 = callExpr
        .flatMap(function (callExpr) { return exports.maybePropertyAccessExpression(callExpr.expression); });
    var prop = prop0
        .flatMap(function (callProp) { return exports.maybeCallExpression(callProp.expression); })
        .flatMap(function (callPropCall) { return exports.maybePropertyAccessExpression(callPropCall.expression); });
    var receiver1 = prop
        .flatMap(function (p) { return exports.maybeIdentifier(p.expression); })
        .map(function (r) { return r.text; });
    var call1 = prop
        .flatMap(function (p) { return exports.maybeIdentifier(p.name); })
        .map(function (r) { return r.text; });
    if (receiver1.filter(function (v) { return v === "angular"; })
        .orElse(call1.filter(function (v) { return v === "module"; })).isSome()) {
        var moduleCall = prop0.map(function (p) { return p.name.text; });
        if (moduleCall.filter(function (v) { return v === "controller"; }).isSome()) {
            var ctrlName = callExpr
                .filter(function (c) { return c.arguments.length > 0; })
                .flatMap(function (c) { return exports.maybeStringLiteral(c.arguments[0]); })
                .map(function (a) { return a.text; });
            var moduleName = prop0
                .flatMap(function (p) { return exports.maybeCallExpression(p.expression); })
                .filter(function (c) { return c.arguments.length > 0; })
                .flatMap(function (c) { return exports.maybeStringLiteral(c.arguments[0]); })
                .map(function (s) { return s.text; });
            var buildModuleCtrl = function (mod, ctrl) { return [mod, ctrl]; };
            return prelude_ts_1.Option.liftA2(buildModuleCtrl)(moduleName, ctrlName);
        }
    }
    return prelude_ts_1.Option.none();
}
function getPropertyByName(objLit, propName) {
    return prelude_ts_1.Option.of(objLit.properties
        .find(function (p) { return exports.maybeIdentifier(p.name).filter(function (i) { return i.getText() === propName; }).isSome(); }));
}
function parseAngularDirectiveTemplate(modelPath, callExpr) {
    var prop0 = exports.maybePropertyAccessExpression(callExpr.expression);
    var prop = prop0
        .flatMap(function (callProp) { return exports.maybeCallExpression(callProp.expression); })
        .flatMap(function (callPropCall) { return exports.maybePropertyAccessExpression(callPropCall.expression); });
    var receiver1 = prop
        .flatMap(function (p) { return exports.maybeIdentifier(p.expression); })
        .map(function (r) { return r.text; });
    var call1 = prop
        .flatMap(function (p) { return exports.maybeIdentifier(p.name); })
        .map(function (r) { return r.text; });
    if (receiver1.filter(function (v) { return v === "angular"; })
        .orElse(call1.filter(function (v) { return v === "module"; })).isSome()) {
        var moduleCall = prop0.map(function (p) { return p.name.text; });
        if (moduleCall.filter(function (v) { return v === "directive"; }).isSome()) {
            var directiveParam = prelude_ts_1.Option.of(callExpr)
                .filter(function (c) { return c.arguments.length > 1; })
                .map(function (c) { return c.arguments[1]; });
            var returnExpr = directiveParam
                .flatMap(exports.maybeArrayLiteralExpression)
                .filter(function (l) { return l.elements.length > 0; })
                .map(function (l) { return l.elements[l.elements.length - 1]; })
                .orElse(directiveParam);
            var arrowBodyExpr = returnExpr
                .flatMap(exports.maybeArrowFunction)
                .flatMap(function (a) { return exports.maybeBlock(a.body); });
            var fnBodyExpr = returnExpr
                .flatMap(exports.maybeFunctionExpression)
                .map(function (fn) { return fn.body; });
            var bodyExpr = arrowBodyExpr.orElse(fnBodyExpr);
            var resultExpr = bodyExpr
                .flatMap(function (b) { return exports.maybeReturnStatement(b.statements[b.statements.length - 1]); })
                .flatMap(function (s) { return prelude_ts_1.Option.of(s.expression); });
            var scopeObject = resultExpr
                .flatMap(exports.maybeAsExpression)
                .map(function (a) { return a.expression; })
                .orElse(resultExpr);
            var templateUrl = scopeObject
                .flatMap(exports.maybeObjectLiteralExpression)
                .flatMap(function (e) { return getPropertyByName(e, "templateUrl"); })
                .flatMap(exports.maybePropertyAssignment)
                .flatMap(function (a) { return exports.maybeStringLiteral(a.initializer); })
                .map(function (s) { return s.text; });
            return templateUrl.map(function (viewPath) { return ({ modelPath: modelPath, viewPath: viewPath }); });
        }
    }
    return prelude_ts_1.Option.none();
}
var modalOpenViewConnector = {
    interceptAstNode: ts.SyntaxKind.CallExpression,
    getControllerView: function (node, projectPath) {
        return parseModalOpen(node).toVector().toArray();
    }
};
var moduleStateViewConnector = {
    interceptAstNode: ts.SyntaxKind.ObjectLiteralExpression,
    getControllerView: function (node, projectPath) {
        return parseModuleState(node).toVector().toArray();
    }
};
var directiveViewConnector = {
    interceptAstNode: ts.SyntaxKind.CallExpression,
    getModelView: function (filename, node, projectPath) {
        return parseAngularDirectiveTemplate(filename, node).toVector().toArray();
    }
};
/**
 * Default set of [[ControllerViewConnector]] which can recognize connections between
 * angular controllers and views from the typescript source.
 * You can give this list in [[ProjectSettings.ctrlViewConnectors]], or you can add
 * your own or provide your own list entirely.
 */
exports.defaultCtrlViewConnectors = [modalOpenViewConnector, moduleStateViewConnector];
/**
 * Default set of [[ModelViewConnector]] which can recognize connections between
 * angular models (contained in controller or directives) and views from the typescript source.
 * You can give this list in [[ProjectSettings.modelViewConnectors]], or you can add
 * your own or provide your own list entirely.
 */
exports.defaultModelViewConnectors = [directiveViewConnector];
/**
 * @hidden
 */
function extractCtrlViewConnsAngularModule(fileName, webappPath, ctrlViewConnectors, modelViewConnectors) {
    var sourceFile = ts.createSourceFile(fileName, fs_1.readFileSync(fileName).toString(), ts.ScriptTarget.ES2016, /*setParentNodes */ true);
    var ngModuleName = prelude_ts_1.Option.none();
    var controllerName = prelude_ts_1.Option.none();
    var controllerViewInfos = [];
    var modelViewInfos = [];
    return new Promise(function (resolve, reject) {
        function nodeExtractModuleOpenAngularModule(node) {
            if (controllerName.isNone() && node.kind == ts.SyntaxKind.ExpressionStatement) {
                var mCtrlNgModule = parseAngularModule(node);
                ngModuleName = mCtrlNgModule.map(function (moduleCtrl) { return moduleCtrl[0]; });
                controllerName = mCtrlNgModule.map(function (moduleCtrl) { return moduleCtrl[1]; });
            }
            controllerViewInfos = controllerViewInfos.concat(prelude_ts_1.Vector.ofIterable(ctrlViewConnectors)
                .filter(function (conn) { return conn.interceptAstNode === node.kind; })
                .flatMap(function (conn) { return prelude_ts_1.Vector.ofIterable(conn.getControllerView(node, webappPath)); })
                .toArray());
            modelViewInfos = modelViewInfos.concat(prelude_ts_1.Vector.ofIterable(modelViewConnectors)
                .filter(function (conn) { return conn.interceptAstNode === node.kind; })
                .flatMap(function (conn) { return prelude_ts_1.Vector.ofIterable(conn.getModelView(fileName, node, webappPath)); })
                .toArray());
            ts.forEachChild(node, nodeExtractModuleOpenAngularModule);
        }
        nodeExtractModuleOpenAngularModule(sourceFile);
        resolve({ fileName: fileName, ngModuleName: ngModuleName, controllerName: controllerName, controllerViewInfos: controllerViewInfos, modelViewInfos: modelViewInfos });
    });
}
exports.extractCtrlViewConnsAngularModule = extractCtrlViewConnsAngularModule;
function nodeIsExported(node) {
    return prelude_ts_1.Option.of(node.modifiers)
        .filter(function (modifiers) { return modifiers.some(function (modifier) { return modifier.kind === ts.SyntaxKind.ExportKeyword; }); })
        .isSome();
}
/**
 * Default set of controller view fragment extractors (currently empty)
 */
exports.defaultCtrlViewFragmentExtractors = [];
/**
 * @hidden
 */
function extractControllerScopeInfo(fileName, ctrlViewFragmentExtractors) {
    var sourceFile = ts.createSourceFile(fileName, fs_1.readFileSync(fileName).toString(), ts.ScriptTarget.ES2016, /*setParentNodes */ true);
    return new Promise(function (resolve, reject) {
        var scopeInfo = prelude_ts_1.Option.none();
        var scopeTypeParams = prelude_ts_1.Option.none();
        var tsModuleName = undefined;
        var typeAliases = [];
        var imports = [];
        var importNames = [];
        var nonExportedDeclarations = [];
        var viewFragments = [];
        function nodeExtractScopeInterface(node) {
            // so that the viewtest file may compile, we must copy
            // in it classes & interfaces that may have been declared
            // privately in the controller. We do limit ourselves to
            // top-level declarations on which the Scope type declaration
            // may depend, that's why we check whether they're under the
            // module block. if there is no TS module and declarations are
            // toplevel then no need to copy them as they were global anyway.
            if (node.parent &&
                node.parent.kind === ts.SyntaxKind.ModuleBlock &&
                !nodeIsExported(node)) {
                if (node.kind === ts.SyntaxKind.InterfaceDeclaration) {
                    var curIntfInfo = parseScopeInterface(node);
                    if (curIntfInfo.isSome()) {
                        scopeInfo = curIntfInfo.map(function (x) { return x.fst(); });
                        scopeTypeParams = curIntfInfo.map(function (x) { return x.snd(); });
                    }
                    else {
                        nonExportedDeclarations.push(node.getText());
                    }
                }
                if (node.kind === ts.SyntaxKind.ClassDeclaration) {
                    nonExportedDeclarations.push(node.getText());
                }
            }
            if (node.kind === ts.SyntaxKind.ModuleDeclaration) {
                var moduleLevel = node.name.text;
                if (tsModuleName) {
                    tsModuleName += "." + moduleLevel;
                }
                else {
                    tsModuleName = moduleLevel;
                }
            }
            if (node.kind === ts.SyntaxKind.TypeAliasDeclaration && !nodeIsExported(node)) {
                typeAliases.push(node.getText());
            }
            if (node.kind === ts.SyntaxKind.ImportEqualsDeclaration) {
                imports.push(node.getText());
                importNames.push(node.name.getText());
            }
            var ctrlViewFragments = prelude_ts_1.Vector.ofIterable(ctrlViewFragmentExtractors)
                .filter(function (extractor) { return extractor.interceptAstNode === node.kind; })
                .flatMap(function (extractor) { return prelude_ts_1.Vector.ofIterable(extractor.getViewFragments(node)); });
            viewFragments = viewFragments.concat(ctrlViewFragments.toArray());
            ts.forEachChild(node, nodeExtractScopeInterface);
        }
        nodeExtractScopeInterface(sourceFile);
        resolve({
            tsModuleName: prelude_ts_1.Option.of(tsModuleName),
            scopeInfo: scopeInfo, scopeTypeParams: scopeTypeParams, typeAliases: typeAliases, imports: imports, importNames: importNames,
            nonExportedDeclarations: nonExportedDeclarations, viewFragments: viewFragments
        });
    });
}
exports.extractControllerScopeInfo = extractControllerScopeInfo;
//# sourceMappingURL=controller-parser.js.map