"use strict";
exports.__esModule = true;
var ts = require("typescript");
var prelude_ts_1 = require("prelude.ts");
var controller_parser_1 = require("./controller-parser");
/**
 * An angular filter. They can be registered through the [[ProjectSettings]] setup.
 * You must give a name, and the type for the filter.
 * Example:
 * ```new NgFilter("translate", "(key: string) => string")```
 *
 * You can also supply a addScopeToParam function: you are given the parameter index,
 * and are called with the parameter, so you can or not add scope accessors.
 * 99% of the time you want scope accessors for filter parameters, but for instance
 * 'filter' allows you to specify a pattern for items through an object literal.
 * If the filter spec is {name: "test"}, we certainly don't want to change it to:
 * {$scope.name: "test"}.
 */
var NgFilter = (function () {
    /**
     * @param name The name of the angular filter
     * @param type The type that'll be used to type-check uses of the filter.
     * @param addScopeToParam Customize the scope adding for parameter index n For instance
     *        for 'filter', we shouldn't wrap the values in the scope for the
     *        'expression' parameter, which is a spec of the values to keep.
     */
    function NgFilter(name, type, addScopeToParam) {
        this.name = name;
        this.type = type;
        // default to add scope accessors if the user didn't specify a custom function
        this.addScopeToParam = addScopeToParam || (function (idx, input, addScopeAccessors) { return addScopeAccessors(input); });
    }
    return NgFilter;
}());
exports.NgFilter = NgFilter;
/**
 * special handling for the expression parameter of the 'filter' filter.
 * that parameter defines a pattern for the items to keep.
 * for instance: {name: 'Peter'}, or {name: myName}
 * We want to change that to {name: $scope.myName}, and certainly not
 * {$scope.name: $scope.myName} and also not {name: myName}.
 * Since we want to 'half' add scope accessors, we need a custom handling.
 */
function filterFilterParams(paramIdx, input, addScAccessors) {
    var normal = addScAccessors(input);
    if (paramIdx != 1) {
        // special handling only for the first parameter of the function
        return normal;
    }
    // if we try to parse "{a:1,b:2}", typescript will parse it as a
    // LabeledStatement, and parses it wrong for us.
    // To force typescript to parse it as an object literal, we change
    // it to "const ___ = {a:1,b:2}"
    var sourceFile = ts.createSourceFile("", "const ___ = " + input, ts.ScriptTarget.ES2016, /*setParentNodes */ true);
    return prelude_ts_1.Option.of(sourceFile)
        .flatMap(function (f) { return controller_parser_1.maybeSingleNode(f.statements); })
        .flatMap(function (st) { return controller_parser_1.maybeVariableStatement(st); })
        .flatMap(function (vs) { return controller_parser_1.maybeSingleNode(vs.declarationList.declarations); })
        .filter(function (decl) { return decl.initializer !== undefined; })
        .flatMap(function (decl) { return controller_parser_1.maybeObjectLiteralExpression(decl.initializer); })
        .flatMap(function (objLit) { return prelude_ts_1.Option.sequence(prelude_ts_1.Vector.ofIterable(objLit.properties.map(controller_parser_1.maybePropertyAssignment))); })
        .map(function (props) { return props.filter(function (p) { return p.initializer !== undefined; }); })
        .map(function (props) { return props.map(function (prop) { return prop.name.getText() + ": " + addScAccessors(prop.initializer.getText()); }); })
        .map(function (props) { return "{" + props.toArray().join(", ") + "}"; })
        .getOrElse(normal);
}
/**
 * Set of angular filters supported out of the box. You can give this list in
 * [[ProjectSettings.ngFilters]], or you can add your own or provide your own
 * list entirely.
 */
exports.defaultNgFilters = [
    new NgFilter("translate", "(key: string) => string"),
    new NgFilter("linky", "(text:string | null, target: '_blank'|'_self'|'_parent'|'_top') => string"),
    new NgFilter("orderBy", "<T, K extends keyof T>(input:T[], field: K) => T[]"),
    new NgFilter("filter", "<T>(input:T[], p: (((v:T,idx:number,array:T[])=>boolean) | string | { [P in keyof T]?: T[P]; })) => T[]", filterFilterParams),
    new NgFilter("limitTo", "<T>(input: T[] | string | number, limit: string|number, begin?: string|number) => T[] | string"),
    new NgFilter("date", "(date:Date|string|number, format?: string, timezone?: string)=>string"),
    new NgFilter("currency", "(amount:number, symbol?: string, fractionSize?:number)=>string")
];
//# sourceMappingURL=filters.js.map