"use strict";
exports.__esModule = true;
var ts = require("typescript");
var prelude_ts_1 = require("prelude.ts");
var P = require("parsimmon");
/**
 * Companion object to assist typescript code generation.
 * It manages the scope behind the scenes so its state
 * changes as you call its methods.
 * If you do not let it know about variables you declare
 * in your typescript, there will be issues of '$scope.'
 * being prepended to the generated code when it shouldn't be.
 */
var CodegenHelper = (function () {
    function CodegenHelper(ngFilters, scope, getNewVarName) {
        var _this = this;
        /**
         * Add scope accessors to a JS expression. For instance,
         * "data.name" will become "$scope.data.name" if the scope
         * has a field named 'data'
         * NOTE using an instance function so this will be properly
         * bound when used as a callback =>
         * https://github.com/Microsoft/TypeScript/wiki/'this'-in-TypeScript#use-instance-functions
         * @param js the javascript from the angular view
         * @returns new source with the scope accessors added
         */
        this.addScopeAccessors = function (js) {
            return addScopeAccessors(_this.ngScopeInfo.soFar.prepend({
                // hardcoding 1...I just need to let addScopeAccessors
                // know about these local variables. a bit of a hack.
                xpathDepth: 1,
                closeSource: function () { return ""; },
                variables: _this.ngScopeInfo.curScopeVars
            }), js);
        };
        this.ngFilters = ngFilters;
        this.ngScopeInfo = { soFar: scope, curScopeVars: [] };
        this.getNewVarName = getNewVarName;
    }
    /**
     * Get a new unique variable name
     * @returns new unique variable name
     */
    CodegenHelper.prototype.getNewVariableName = function () {
        return this.registerVariable(this.getNewVarName());
    };
    /**
     * Generate a TS expression declaring a variable of
     * the type and value that you give. Will automatically call
     * `addScopeAccessors` on the value.
     * @param type typescript type for the variable
     * @param val value for the variable
     * @returns typescript expression that registers the variable, as string.
     */
    CodegenHelper.prototype.declareVariable = function (type, val) {
        // if there are embedded {{}} blocks, ignore this and we'll grab them
        // in the html source in general through other means.
        if (val.length > 0 && val.indexOf("{{") < 0) {
            return "const " + this.getNewVariableName() + ": " + type + " = " + this.addScopeAccessors(val) + ";";
        }
        else {
            return ""; // angular tolerates empty attributes and ignores them, for instance ng-submit=""
        }
    };
    /**
     * You must register a variable name when you declare a variable
     * while generating code without going through [[generateVariable]]
     * or [[getNewVariableName]].
     * Otherwise generation will add a `$scope.` accessor to it even though
     * it shouldn't.
     * Since `registerVariable` will return you the variable name you gave,
     * you can use this function as a pass-through, just wrap your var
     * name with this call.
     */
    CodegenHelper.prototype.registerVariable = function (name) {
        this.ngScopeInfo.curScopeVars.push(name);
        return name;
    };
    return CodegenHelper;
}());
exports.CodegenHelper = CodegenHelper;
/**
 * @hidden
 */
function keyword(txt) {
    return P.whitespace.then(P.string(txt)).skip(P.whitespace);
}
exports.keyword = keyword;
/**
 * @hidden
 */
function parseAtom() {
    return P.takeWhile(function (c) { return [' ', '|'].indexOf(c) < 0; });
}
exports.parseAtom = parseAtom;
function parseNgExpr() {
    return P.string("{{").then(P.lazy(parseExpr)).skip(P.string("}}"));
}
function parseArithmeticOperator() {
    return keyword("+").or(keyword("-")).or(keyword("*")).or(keyword("/"));
}
function parseLogicalOperator() {
    return keyword("&&").or(keyword("||"))
        .or(keyword("===")).or(keyword("!=="))
        .or(keyword("<")).or(keyword("<=")).or(keyword(">")).or(keyword(">="));
}
function parseBinaryOperations() {
    return parseString().or(parseAtom())
        .chain(function (expr) { return parseLogicalOperator().or(parseArithmeticOperator())
        .chain(function (op) { return parseBinaryOperations().or(parseString()).or(parseAtom())
        .map(function (expr2) { return expr + op + expr2; }); }); });
}
function parseTernary() {
    return parseBinaryOperations().or(parseAtom())
        .skip(keyword("?"))
        .chain(function (expr) { return parseExpr()
        .chain(function (expr2) { return keyword(":").then(parseExpr())
        .map(function (expr3) { return expr + " ? " + expr2 + ":" + expr3; }); }); });
}
function parseString() {
    var str = function (sep) { return P.string(sep)
        .then(P.noneOf(sep).many())
        .skip(P.string(sep))
        .map(function (s) { return sep + s.join("") + sep; }); };
    return str("'").or(str('"'));
}
function parseBracketed(next) {
    return P.regexp(/\s*\(\s*/).then(next).skip(P.regexp(/\s*\)/));
}
function parseExpr() {
    return parseNgExpr()
        .or(parseBracketed(P.lazy(parseExpr)))
        .or(parseTernary())
        .or(parseBinaryOperations())
        .or(parseString())
        .or(parseAtom());
}
/**
 * [Parsimmon](https://github.com/jneen/parsimmon) parser for angular filter
 * expressions. You can then use [[ngFilterExpressionToTypeScriptEmbedded]]
 * and [[ngFilterExpressionToTypeScriptStandalone]] to operate on the data.
 * @returns a [Parsimmon](https://github.com/jneen/parsimmon) Parser of [[NgFilterExpression]]
 */
function parseNgFilterExpression() {
    return P.optWhitespace.then(parseExpr())
        .chain(function (expression) { return P.regex(/\s*\|\s*/).then(parseNgFilterCall()).many()
        .map(function (filterCalls) { return ({ expression: expression, filterCalls: filterCalls }); }); });
}
exports.parseNgFilterExpression = parseNgFilterExpression;
function parseNgFilterCall() {
    return P.takeWhile(function (c) { return [' ', ':', ')'].indexOf(c) < 0; }).chain(function (functionName) { return parseNgFilterParam().many()
        .map(function (functionParameters) { return ({ functionName: functionName, functionParameters: functionParameters }); }); });
}
function parseNgFilterParam() {
    var simpleParam = P.takeWhile(function (c) { return [':', '|'].indexOf(c) < 0; });
    var objectLiteralParam = P.string("{")
        .then(P.takeWhile(function (c) { return c !== "}"; })).skip(P.string("}")).map(function (s) { return "{" + s + "}"; });
    return P.regex(/\s*:\s*/).then(objectLiteralParam.or(simpleParam));
}
function wrapFilterCall(ngFilters, addScAccessors) {
    return function (soFar, ngFilterCall) {
        var filterInfo = ngFilters.find(function (f) { return f.name === ngFilterCall.functionName; }).getOrUndefined();
        if (!filterInfo) {
            throw "Unknown filter: " + ngFilterCall.functionName + " -- context: " + soFar;
        }
        var addAccessorsForParam = filterInfo.addScopeToParam;
        var params = ngFilterCall.functionParameters
            .map(function (val, idx) { return addAccessorsForParam(idx + 1, val, addScAccessors); }).join(', ');
        var fnParams = params.length > 0 ? (', ' + params) : '';
        return "f__" + ngFilterCall.functionName + "(" + soFar + fnParams + ")";
    };
}
/**
 * Convert an angular filter expression to typescript code.
 * For instance, "data.items | orderBy: 'name'" will become:
 * "f___orderBy($scope.data.items, 'name');".
 * Calls [[ngFilterExpressionToTypeScriptStandalone]] under the hood.
 * @param expr The angular filter expression
 * @param codegenHelpers Object which contains functions
 *     to assist with typescript code generation.
 * @returns A typescript expression for type-checking the angular filters,
 *     or the empty string in case of parse error.
 */
function filterExpressionToTypescript(expr, codegenHelpers) {
    if (expr.indexOf("|") < 0) {
        return ngFilterExpressionToTypeScriptStandalone({ expression: expr, filterCalls: [] }, codegenHelpers);
    }
    var ngFilterExpr = parseNgFilterExpression().skip(P.optWhitespace).parse(expr);
    if (!ngFilterExpr.status) {
        console.warn("Failed parsing filter expression");
        console.warn(expr);
        console.warn(ngFilterExpr);
        return "";
    }
    return ngFilterExpressionToTypeScriptStandalone(ngFilterExpr.value, codegenHelpers);
}
exports.filterExpressionToTypescript = filterExpressionToTypescript;
/**
 * Convert a parsed angular filter expression to typescript code.
 * For instance, "data.items | orderBy: 'name'" will become:
 * "f___orderBy($scope.data.items, 'name');".
 * @param ngFilterExpr The parsed angular filter expression
 * @param codegenHelpers Object which contains functions
 *     to assist with typescript code generation.
 * @returns A typescript expression for type-checking the angular filters,
 *     or the empty string in case of parse error.
 */
function ngFilterExpressionToTypeScriptStandalone(ngFilterExpr, codegenHelpers) {
    if (ngFilterExpr.filterCalls.length === 0) {
        return codegenHelpers.declareVariable("any", ngFilterExpr.expression);
    }
    return ngFilterExpr.filterCalls.reduce(wrapFilterCall(codegenHelpers.ngFilters, codegenHelpers.addScopeAccessors), codegenHelpers.addScopeAccessors(ngFilterExpr.expression)) + ";";
}
exports.ngFilterExpressionToTypeScriptStandalone = ngFilterExpressionToTypeScriptStandalone;
/**
 * Convert a parsed angular filter expression to typescript code.
 * For instance, "data.items | orderBy: 'name'" will become:
 * "f___orderBy($scope.data.items, 'name')".
 * Unlike [[ngFilterExpressionToTypeScriptStandalone]], this version will
 * generate typescript code to be reused by further code, not to be generated
 * standalone. For instance:
 * `ng-options="item.subItem as item.label for item in data.groups | orderBy:'labelSort'"`
 * In that case we can generate a typescript embeddable expression for:
 * `data.groups | orderBy:'labelSort'` and then include it in the rest of the
 * outer expression.
 * @param ngFilterExpr The parsed angular filter expression
 * @param codegenHelpers Code generation helpers
 * @returns A typescript expression for type-checking the angular filters,
 *     or the empty string in case of parse error.
 */
function ngFilterExpressionToTypeScriptEmbedded(ngFilterExpr, codegenHelpers) {
    if (ngFilterExpr.filterCalls.length === 0) {
        return codegenHelpers.addScopeAccessors(ngFilterExpr.expression);
    }
    return ngFilterExpr.filterCalls.reduce(wrapFilterCall(codegenHelpers.ngFilters, codegenHelpers.addScopeAccessors), codegenHelpers.addScopeAccessors(ngFilterExpr.expression));
}
exports.ngFilterExpressionToTypeScriptEmbedded = ngFilterExpressionToTypeScriptEmbedded;
/**
 * @hidden
 */
function addScopeAccessors(scopes, _input) {
    // ugly trick of prepending "!" in case the first character is "{"
    // the reason is that without that, the typescript parser interprets
    // as a Block. In reality it's an object literal. I found out that if
    // if it starts with !{ instead, then the typescript parser correctly
    // treats it as an object literal.
    var input = _input.trim().startsWith("{") ? "!" + _input.trim() : _input;
    var sourceFile = ts.createSourceFile("", input, ts.ScriptTarget.ES2016, /*setParentNodes */ true);
    var result = sourceFile.statements.map(stmtAddScopeAccessors(scopes)).join("");
    // remove the leading ! if we added one.
    return _input.trim().startsWith("{") ? result.substring(1) : result;
}
exports.addScopeAccessors = addScopeAccessors;
function handleRegexpNode(node) {
    // {} and other characters in regex literals confuse the indenting
    // pass that I have during codegen. generate a more uniform syntax.
    if (node.text.startsWith('/') && node.text.endsWith('/')) {
        var regexText = node.text.substring(1, node.text.length - 1);
        var separator = '"';
        if (node.text.indexOf('"') >= 0) {
            separator = "'";
            if (node.text.indexOf("'") >= 0) {
                // ok that's probably too much now.
                // return in the // form. if there are characters
                // which could confuse the layout pass, maybe this
                // can be worked around (put the regex in the controller,
                // not the view). or I can try yet another way but...
                if (node.text.indexOf('{') >= 0 ||
                    node.text.indexOf('}') >= 0 ||
                    node.text.indexOf(';') >= 0) {
                    console.warn("Warning: it's likely that the rendering of the regular expression" +
                        regexText + " causes problems." +
                        " Consider moving its value to the view instead of the controller" +
                        " or changing it to the 'new Regexp()' form.");
                }
                return node.getText();
            }
        }
        return "new RegExp(" + separator + regexText + separator + ")";
    }
    else {
        return node.getText();
    }
}
var nodeKindPassthroughList = prelude_ts_1.HashSet.of(ts.SyntaxKind.NumericLiteral, ts.SyntaxKind.NullKeyword, ts.SyntaxKind.StringLiteral, ts.SyntaxKind.TrueKeyword, ts.SyntaxKind.FalseKeyword, ts.SyntaxKind.UndefinedKeyword);
function stmtAddScopeAccessors(scopes) {
    return function (node) {
        if (node.kind === ts.SyntaxKind.ExpressionStatement) {
            return stmtAddScopeAccessors(scopes)(node.expression);
        }
        else if (node.kind === ts.SyntaxKind.RegularExpressionLiteral) {
            return handleRegexpNode(node);
        }
        else if (node.kind === ts.SyntaxKind.PropertyAccessExpression) {
            var prop = node;
            return stmtAddScopeAccessors(scopes)(prop.expression) + "." + prop.name.getText();
        }
        else if (node.kind === ts.SyntaxKind.Identifier) {
            return node.getText() === "undefined" ? "undefined" : addScopePrefixIfNeeded(scopes, node.getText());
        }
        else if (node.kind === ts.SyntaxKind.PrefixUnaryExpression) {
            var op = node;
            return ts.tokenToString(op.operator) + stmtAddScopeAccessors(scopes)(op.operand);
        }
        else if (node.kind === ts.SyntaxKind.CallExpression) {
            var expr = node;
            return stmtAddScopeAccessors(scopes)(expr.expression) + "(" +
                expr.arguments.map(stmtAddScopeAccessors(scopes)).join(", ") + ")";
        }
        else if (node.kind === ts.SyntaxKind.BinaryExpression) {
            var expr = node;
            return stmtAddScopeAccessors(scopes)(expr.left)
                + " " + expr.operatorToken.getText() + " "
                + stmtAddScopeAccessors(scopes)(expr.right);
        }
        else if (node.kind === ts.SyntaxKind.ElementAccessExpression) {
            var acc = node;
            var argValue = acc.argumentExpression
                ? stmtAddScopeAccessors(scopes)(acc.argumentExpression)
                : "";
            return stmtAddScopeAccessors(scopes)(acc.expression) +
                "[" + argValue + "]";
        }
        else if (node.kind === ts.SyntaxKind.ConditionalExpression) {
            var cond = node;
            return stmtAddScopeAccessors(scopes)(cond.condition) + " ? " +
                stmtAddScopeAccessors(scopes)(cond.whenTrue) + " : " +
                stmtAddScopeAccessors(scopes)(cond.whenFalse);
        }
        else if (node.kind === ts.SyntaxKind.LabeledStatement) {
            var lStat = node;
            return lStat.label.text + ": " + stmtAddScopeAccessors(scopes)(lStat.statement);
        }
        else if (node.kind === ts.SyntaxKind.SyntaxList) {
            return node.getChildren().map(stmtAddScopeAccessors(scopes)).join("");
        }
        else if (nodeKindPassthroughList.contains(node.kind)) {
            return node.getText();
        }
        else if (node.kind >= ts.SyntaxKind.FirstToken && node.kind <= ts.SyntaxKind.LastToken) {
            return ts.tokenToString(node.kind);
        }
        else if (node.kind === ts.SyntaxKind.ParenthesizedExpression) {
            return "(" + stmtAddScopeAccessors(scopes)(node.expression) + ")";
        }
        else if (node.kind === ts.SyntaxKind.ArrayLiteralExpression) {
            return "[" + node.elements.map(stmtAddScopeAccessors(scopes)).join(", ") + "]";
        }
        else if (node.kind === ts.SyntaxKind.ObjectLiteralExpression) {
            return "{" +
                node.properties
                    .map(stmtAddScopeAccessors(scopes)).join(", ") +
                "}";
        }
        else if (node.kind === ts.SyntaxKind.PropertyAssignment) {
            var paNode = node;
            return paNode.name.getText() + ": " + stmtAddScopeAccessors(scopes)(paNode.initializer);
        }
        console.log("Add scope accessors: unhandled node: " + node.kind + " -- " + node.getText());
        return node.getText();
    };
}
function addScopePrefixIfNeeded(scopes, expression) {
    // extract the field name from the expression, which can be...
    // data.user.getName(), or getName() or things like that.
    // so we stop at the first "." or "(" to get respectively
    // "data" or "getName".
    var fieldName = expression.replace(/[\(\.].*$/, "");
    // is the field name present in any of the parent scopes?
    if (scopes.find(function (s) { return s.variables.indexOf(expression) >= 0; }).isSome()) {
        // YES => read it from there.
        return expression;
    }
    else {
        return "$scope." + expression;
    }
}
//# sourceMappingURL=view-ngexpression-parser.js.map